/**
 * servicetrade-portal-admin-add-user.js
 * Handles adding a user in the service trade admin section.
 * 
 * TODO:
 * Enabling/Disabling of form
 * Error message on error
 */
(function( $ ) {
	'use strict';

  var buildUrl = function(path) {
    return window.servicetradePortalConfig ? window.servicetradePortalConfig.apiRoot + path : path;
  };

  /**
   * Converts a array of objects with name value pairs to an object.
   * @param {*} unindexed_array 
   */
  var arrayToObject = function (unindexed_array){
    var indexed_array = {};
    $.map(unindexed_array, function(n, i){
        indexed_array[n['name']] = n['value'];
    });
    return indexed_array;
  }

  /**
   * Converts DOM idiosyncrasies to clean object representation.
   * @param {Object} formObject 
   */
  var transformToWire = function(formObject) {
    formObject.featureRequestServiceEnabled = formObject.featureRequestServiceEnabled === 'on';
    formObject.featureViewQuotesEnabled = formObject.featureViewQuotesEnabled === 'on';
    formObject.featureViewDeficienciesEnabled = formObject.featureViewDeficienciesEnabled === 'on';
    formObject.featureViewPricesEnabled = formObject.featureViewPricesEnabled === 'on';

    return formObject;
  }



  /**
   * Performs client side validation.
   * @param {Object} wireObject 
   */
  var validateData = function(wireObject) {
    var result = {
      isValid: wireObject.email.trim().length >= 4 && // There exists no good regex for email
               wireObject.username.trim().length >= 1 && 
               wireObject.password.trim().length >= 8 && // Just some basic verification
               wireObject.password === wireObject.verifyPassword,
      errors: {}
    };
    if(wireObject.password !== wireObject.verifyPassword) {
      result.errors.verifyPassword = "Passwords don't match.";
    }

    if(wireObject.email.trim().length < 4) {
      result.errors.email = "Please enter an email.";
    }

    if(wireObject.username.trim().length < 1) {
      result.errors.username = "Please enter a username.";
    }

    if(wireObject.password.trim().length < 8) {
      result.errors.password = "Please enter a password with at least 8 characters.";
    }

    return result;
  }

  var resetErrorSuccessDisplay = function() {
    var $formSuccess = $('.servicetrade-page-admin-add-user .form-success');
    $formSuccess.hide();
    
  };
  
  var disableAllInputs = function() {
    $('.servicetrade-page-admin-add-user input').prop('disabled',true);
  };

  var enableAllInputs = function() {
    $('.servicetrade-page-admin-add-user input').prop('disabled',false);    
  };

  /**
   * Updates the form and sets/clears the error fields.
   * @param {Object} errorsObject 
   */
  var displayValidationErrors = function(errors) {
    $('.servicetrade-page-admin-add-user .servicetrade-field-email-error').text(errors.email || '');
    $('.servicetrade-page-admin-add-user .servicetrade-field-username-error').text(errors.username || '');
    $('.servicetrade-page-admin-add-user .servicetrade-field-password-error').text(errors.password || '');
    $('.servicetrade-page-admin-add-user .servicetrade-field-verify-password-error').text(errors.verifyPassword || '');
    $('.servicetrade-page-admin-add-user .form-error').text(errors.form);

    var hasError = errors.email || errors.username || errors.password || errors.verifyPassword || errors.form;


    var $form = $('.servicetrade-page-admin-add-user');
    $form.toggleClass('serviceportal-error',hasError).removeClass('serviceportal-success');
  }

  var onAJAXSuccess = function(data) {
    enableAllInputs();
    var $form = $('.servicetrade-page-admin-add-user');
    var $formSuccess = $('.servicetrade-page-admin-add-user .form-success');
    $('.servicetrade-page-admin-add-user input').val('');
    $('.servicetrade-page-admin-add-user input:first').focus();
    
    // This must be called after the val because that triggers a blur

    $formSuccess.text("Successfully added user!").show();
    $form.removeClass('serviceportal-error').addClass('serviceportal-success');
  };

  var onAJAXFail = function(error) {
    enableAllInputs();
    // displayValidationErrors(error.data.errors);
    
    // "data":{"status":422}}
    console.log(error.responseJSON);

    displayValidationErrors( {form: error.responseJSON.message});
  }

  var onChangeAddUserForm = function(e) {
    var formData = $(e.target).closest('form').find(":input").serializeArray();
    formData = arrayToObject(formData);
    formData = transformToWire(formData);
    var validationResult = validateData(formData);
    $(e.target).closest('form').find('button[type="submit"]').prop('disabled', !validationResult.isValid);

    resetErrorSuccessDisplay();
    // console.log(validationResult.errors);
    displayValidationErrors(validationResult.errors);
  };

  var onSubmitAddUserForm = function(e) {
    e.preventDefault();

    var formData = $(e.target).find(":input").serializeArray();
    formData = arrayToObject(formData);
    formData = transformToWire(formData);
    var validationResult = validateData(formData);

    if(validationResult.isValid) {
      // console.log(formData); //use the console for debugging, F12 in Chrome, not alerts

      disableAllInputs();
      $.ajax({
          type: 'POST',
          data: formData,
          url: buildUrl('users'),
          dataType: 'json',
          encode: true,
          beforeSend: ( xhr ) => {
            if (window.servicetradePortalConfig &&
                window.servicetradePortalConfig.nonce) {
              xhr.setRequestHeader( 'X-WP-Nonce', window.servicetradePortalConfig.nonce );
            }
          }
        })
      .done(onAJAXSuccess)
      .fail(onAJAXFail);
    } else {
      displayValidationErrors(validationResult.errors)
    }
  }

  $( document ).ready(function() {
    $('form.servicetrade-form-admin-add-user input').change(onChangeAddUserForm);
    $('form.servicetrade-form-admin-add-user input').keyup(onChangeAddUserForm);
    $('form.servicetrade-form-admin-add-user').submit(onSubmitAddUserForm);
  });
})( jQuery );
