<?php

require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-action.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-helper-sideloadmapper.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'public/rest/class-current-timezone.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'public/rest/class-public-rest-action.php';

/**
 * Returns upcoming jobs
 *
 * An action class.
 *
 * @link       http://www.servicetrade.com
 * @since      2.0.0
 * @package    Servicetrade_Service_Portal
 * @subpackage Servicetrade_Service_Portal/public/rest/actions
 * @author     ServiceTrade <support@servicetrade.com>
 * @example    
 *						 curl http://localhost:8888/wp-json/servicetrade-service-portal/v2/public/jobs/upcoming
 */
class Servicetrade_Service_Portal_Public_Actions_Rest_Jobs_Upcoming_Get_Action extends Servicetrade_Service_Portal_Public_Rest_Action {

  private $filterLocationId;
  private $filterServiceLineId;
  private $filterJobType;
  
	public function __construct($filterLocationId, 
															$filterServiceLineId, 
															$filterJobType) {
    $this->filterLocationId = $filterLocationId;
		$this->filterServiceLineId = $filterServiceLineId;
    $this->filterJobType = $filterJobType;
  }

  function run() {
		$timezone = Servicetrade_Service_Portal_Public_Current_Timezone::timezone();
    $user = $this->get_authenticated_user();
    if(null === $user) {
			return $this->create_rest_error_not_authorized();
    }

	 	$contactId = $user->getContactId();

		$contextKey = null;
		$contextVals = [];
		if ( $user->isCompanyOnlyContact() ) {
			$contextVals = $user->getContactCompanies( true );
			$contextKey = 'customerId';
		} else {
			$contextVals = $user->getContactLocations( true );
			$contextKey = 'locationId';
		}

		if (!$contextKey || !$contextVals) {
			$result =  array(
				jobs => [],
				status => 'no_associated_contacts'
			);
			return new WP_REST_Response( $result, 200 );

			// return new WP_Error( 'servicetrade_failed_request' ,'The plugin could not retrieve data from the servicetrade API', array( 'status' => 503 ) );
		}

		$query = [
			'windowBeginsAfter' => $timezone->tomorrowStart,
			$contextKey => $contextVals,
		];
		$upcomingJobs = $this->get_st_appointments_by_query( $query );

		/*
		Result Mapping - this should be refactored into a common base class (eventually)
		*/

		$mappedResults = array();

		foreach ($upcomingJobs as $result) {

			$mappedServiceLines = array();
         $mappedServiceDescriptions = array();

			foreach ($result['serviceRequests'] as $serviceRequest) {
				if ($serviceRequest['description'] && array_intersect(['public', 'customer'], $serviceRequest['visibility'])) {
					$mappedServiceDescriptions[] = $serviceRequest['description'];
				}
				if( isset($serviceRequest['serviceLine'])) {
					$mappedServiceLines[$serviceRequest['serviceLine']['id']] = array(
						'id' => $serviceRequest['serviceLine']['id'],
						'name' => $serviceRequest['serviceLine']['name'],
					);
				}
			}

			// we use SL IDs to ensure a unique list of SLs, but now the array needs to be reindexed
			$mappedServiceLines = array_values($mappedServiceLines);

			$invoiceAmount = 0;
			foreach ($result['job']['invoices'] as $invoice) {
				if(!empty($invoice['totalPrice'])) {
					$invoiceAmount = $invoiceAmount  + $invoice['totalPrice'];
				}
			}


			$isOnsite = false;
			$isEnroute = false;
			$onsiteStartTime = null;
			$enrouteStartTime = null;
			
			if($result['status'] === 'scheduled') {
				if(! empty($result['clockEvents']) ) {
					foreach ($result['clockEvents'] as $clock_event) {
						if($clock_event['type'] === 'clock-in' &&
							$clock_event['activity'] === 'onsite') {
								$isOnsite = true;
								$onsiteStartTime = $clock_event['time'];
						} else if($clock_event['type'] === 'clock-out' &&
							$clock_event['activity'] === 'onsite') {
							$isOnsite = false;
						} else if($clock_event['type'] === 'clock-in' &&
							$clock_event['activity'] === 'enroute') {
								$isEnroute = true;
								$enrouteStartTime = $clock_event['time'];
						} else if($clock_event['type'] === 'clock-out' &&
							$clock_event['activity'] === 'enroute') {
							$isEnroute = false;
						}
					}
				}
			}

			// FILTER

			if(!empty($this->filterLocationId)) {
				if($this->filterLocationId != $result['job']['location']['id']) {
					continue;
				}
			}

			if(!empty($this->filterJobType)) {
				if($this->filterJobType != $result['job']['type']) {
					continue;
				}
			}

			if(!empty($this->filterServiceLineId)) {
				$foundServiceLine = false;

				foreach($mappedServiceLines as $serviceLine) {
					if($this->filterServiceLineId == $serviceLine['id']) {
						$foundServiceLine = true;
					}
				}

				if(!$foundServiceLine) {
					continue;
				}
			}

			$serviceLinkData = ['jobId' => $result['job']['id'], 'contactId' => $contactId];
			$encryptedServiceLinkData = Servicetrade_Service_Portal_Public_Authenticated_User::encryptUserData($serviceLinkData);

			$jobDescription = in_array('description', $result['job']['visibility']) ? $result['job']['description'] : '';
			if (!$jobDescription) {
				$jobDescription = implode('<br />', $mappedServiceDescriptions);
			}

			$mappedResults[] = array(
				'id' => $result['id'],
				'uri' => $result['uri'],
				'serviceLinkData' => $encryptedServiceLinkData,
				'description' => $jobDescription,
				'locationName' => $result['job']['location']['name'],
				'locationId' => $result['job']['location']['id'],
				'windowStart' => $result['windowStart'],
				'jobId' => $result['job']['id'],
				'jobType' => $result['job']['type'],
				'jobNumber' => $result['job']['number'],
				'jobTypeDisplay' => ucwords( str_replace("_", " ", $result['job']['type'])),
				'serviceLines' => $mappedServiceLines,
				'invoiceAmount' => $invoiceAmount,
				'isOnsite' => $isOnsite,
				'isEnroute' => $isEnroute
			);
			
		}

    $result =  array(
			"jobs" => $mappedResults,
		);
		return new WP_REST_Response( $result, 200 );
	} 

		/**
	 * Find appointments by a query from ServiceTrade API
	 * @since     1.0.0
	 * @return    array    Array of ServiceTrade appointments arrays
	 */
	 public function get_st_appointments_by_query($query=[]) {
		
		$params = http_build_query($query);
		$params .= '&_sideload=appointment.job,job.location,job.deficiencies,appointment.clockEvents,appointment.serviceRequests,serviceRequest.serviceLine,job.invoices';
		$data = $this->st_api()->get( '/appointment?'.$params );
	
		if ( $data['appointments'] ) {
			$mapParams = [
				'job'             => 'jobs',
				'location'        => 'locations',
				'clockEvents'     => 'clockEvents',
				'serviceRequests' => 'serviceRequests',
				'serviceLine'     => 'serviceLines',
				'deficiencies'    => 'deficiencies',
				'invoices'        => 'invoices',
			];
			$mappedData =  Servicetrade_Helper_Sideloadmapper::map_sideloaded_data( $data, 'appointments', $mapParams );

			return $mappedData;
		}
		return [];
	}
}
