<?php

require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-authenticated-api.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-helper-sideloadmapper.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-user-meta.php';

define('ST_CIPHER', 'AES-128-CBC');
define('ST_SHA2LEN', 32);
define('ST_HASH_ALGORITHM', 'sha256');

/**
 * Represents an authenticated user with contact
 *
 * Used to retrieve the current user from wordpress and figure out
 * the associated contact.
 *
 * @link       http://www.servicetrade.com
 * @since      2.0.0
 * @package    Servicetrade_Service_Portal
 * @subpackage Servicetrade_Service_Portal/public/rest
 * @author     ServiceTrade <support@servicetrade.com>
 * @example
 */

class Servicetrade_Service_Portal_Public_Authenticated_User {


  public $email = null;
  /*
  The single consolidated contact for this email.
  Can be null.
  */
  public $contact = null;

  /**
  * Returns the authenticated user, or null if no user is authenticated.
  */
  public static function user() {

    $currentWordpressUser = wp_get_current_user();
    // echo  $currentWordpressUser->ID;
    if ( !($currentWordpressUser instanceof WP_User) ) {
      return null;
    }

    /**
     * Load meta data info for current wordpress user and check if
     * that user has access to data.
     */


    $user = new Servicetrade_Service_Portal_Public_Authenticated_User();
    $user->email = $currentWordpressUser->user_email;

    if(null !== $user->email) {
      $user->contact = Servicetrade_Service_Portal_Public_Authenticated_User::get_contact_for_email($user->email, $currentWordpressUser->ID);
    }
    return $user;
  }

  /**
	* Encodes a JSON object of user data for transmission over insecure HTTP requests
	* Use cases: service link and quote view redirection
	*
	* @TODO MAKE A BETTER KEY!
	*
	* @param array $data
	* @return string
	*/
  public static function encryptUserData($data) {
	  $json = json_encode($data);
	  $encrypted = self::encrypt($json, 'userdata');
	  return $encrypted;
  }

  public static function decryptUserData($dataString) {
	  $json = self::decrypt($dataString, 'userdata');
	  $data = json_decode($json, true);
	  return $data;
  }

  public function contactHasCompanies() {
    return $this->contact && $this->contact['companies'];
  }

  public function contactHasLocations() {
    return $this->contact && $this->contact['locations'];
  }

  public function getContactCompanies($idOnly = false) {
	  $companies = $this->contact ? $this->contact['companies'] : [];
	  if ($idOnly) {
		 $companies = array_map(function($c) { return $c['id']; }, $companies);
	  }
	  return $companies;
  }

  public function getContactLocations($idOnly = false) {
	  $locations = $this->contact ? $this->contact['locations'] : [];
	  if ($idOnly) {
		 $locations = array_map(function($l) { return $l['id']; }, $locations);
	  }
	  return $locations;
  }

  public function isCompanyOnlyContact() {
	  return $this->contact ? $this->contact['companyOnly'] : false;
  }

  public function getContactId() {
	  return $this->contact ? $this->contact['id'] : null;
  }

  public function contactCustomerId() {
    return implode(",", array_map(function($company) { return $company['id']; }, $this->contact['companies']));    
  }

  public function contactLocationId() {
    return implode(",", array_map(function($location) { return $location['id']; }, $this->contact['locations']));
  }

  /*
  private static function currentEmailFromWordpressUser() {
    $currentUser = wp_get_current_user();
    if($currentUser) {
      return $currentUser->user_email;
    } else {
      return null;
    }

    // return "services+nauti@servicetrade.com"; // Company
    // return "services+demo@servicetrade.com"; // Company + Location
  }
  */

  /**
	 * Retrieve a consolidated single contact object for an email
	 * 
	 * This call is incredibly expensive. Because of that we need to cache the results in the local user meta object in wordpress
	 * and renew it every 24 hours or so. It also needs to be manually renewed whenever the user logs in/out in the admin interface.
	 * 
	 *
	 * @since     1.0.0
	 * @param     string  $email
	 * @return    array   modified ST contact object with lists of companies and locations, and extra flags
	 */
  private static function get_contact_for_email($email, $userId) {

		// Step 1 - we retrieve the cached data, if any.
		
		try {
  		$cached_location = Servicetrade_User_Meta::get_locations_cache($userId);
  	
  		if(isset($cached_location)) {
  			$result = Servicetrade_Service_Portal_Public_Authenticated_User::handle_st_contacts_sideload($cached_location);


				if( is_wp_error($result) ) {
					throw $result;
				}
				
  			return $result;
  		}
		} catch (Exception $e) {
			error_log('Failed to deserialize cached_location: ' .  $e->getMessage() . "\n");
		}

  	
  	// No cached data

		$contactData = Servicetrade_Service_Portal_Public_Authenticated_User::get_st_contacts_by_email_sideload($email);
		$contact = $contactData ? Servicetrade_Service_Portal_Public_Authenticated_User::handle_st_contacts_sideload($contactData) : null;
		
		if(isset($contactData)) {
			Servicetrade_User_Meta::set_locations_cache($userId,  $contactData);
		}
		
		return $contact;
  }

  /**
	 * Find contacts by their email address from ServiceTrade API.  Returns data includes a set of companies and locations that are related to each contact -- either directly, through contact->location relationship, or indirectly through contact->company->location relationship
	 *
	 * @since     1.0.0
	 * @return    array    ServiceTrade contact array
	 */
  private static function get_st_contacts_by_email_sideload($email) {
		$data = Servicetrade_Authenticated_API::api()->get( '/contact?_sideload=contact.locations,contact.companies,company.locations&limit=100&search=' . $email );
		return $data['contacts'] ? $data : [];
	}

	/**
	 * Turns list of contacts into a something our customer job view can consume.
	 *
	 * @TODO this is way too complex. We ought to be able to just collapse all the contacts' locations into one big list and
	 * just return it, but this code still has vestiges of back when contacts had a single association to location or company
	 *
	 * @since    1.0.0
	 * @param    array   $contacts - set of ST API contacts
	 * @return   array   modified $contact with lists of company and location data, and extra flags
	 */
   private  static function handle_st_contacts_sideload( $contactData )
	{
		$contact = null;

		if ( $contactData && $contactData['contacts'] ) {
			$contact = $contactData['contacts'][0];
			$contact['locations'] = [];
			$contact['companies'] = $contactData['companies'] ?: [];
		}

		$mappedData = $contactData['contacts'];

		// get IDs of all locations directly associated with contacts
		$locationIds = [];
		foreach($mappedData as $c) {
			$locationIds = array_merge($locationIds, $c['locations']);
		}

		// get IDs of all companies directly associated with contacts
		$companyIds = array_map(function($c) {
			return $c['id'];
		}, $contact['companies']);


		// collect locations indirectly associated through associated companies
		$locationsForCompanyContacts = array_filter($contactData['locations'], function( $l ) use ( $companyIds ) {
			return in_array( $l['company'], $companyIds );
		});

		// collect all directly associated locations
		$locationsForLocationContacts = array_filter($contactData['locations'], function( $l ) use ( $locationIds ) {
			return in_array( $l['id'], $locationIds );
		});

		$contact['locations'] = array_merge( $locationsForLocationContacts, $locationsForCompanyContacts );
		$contact['locations'] = Servicetrade_Service_Portal_Public_Authenticated_User::unique_multi( $contact['locations'] );
		$contact['companies'] = Servicetrade_Service_Portal_Public_Authenticated_User::unique_multi( $contact['companies'] );

		if (!$contact['locations']) {
			unset( $contact['locations'] );
		}
		if (!$contact['companies']) {
			unset( $contact['companies'] );
		}

		$contact['companyOnly'] = (bool)($companyContacts && !$locationContacts);

		return $contact;
   }



   	/**
	 * Dedupes a multidimensional array
	 *
	 *
	 * @since     1.0.0
	 * @param     array
	 * @return    array
	 */
	public static function unique_multi($array) {
    return array_map("unserialize",
      array_unique(array_map("serialize", $array))
    );
	}

	private static function encrypt($input_string, $key){
		if (extension_loaded('openssl')) {
			$ivlen = openssl_cipher_iv_length(ST_CIPHER);
			$iv = openssl_random_pseudo_bytes($ivlen);
			$ciphertext_raw = openssl_encrypt($input_string, ST_CIPHER, $key, OPENSSL_RAW_DATA, $iv);
			$hmac = hash_hmac(ST_HASH_ALGORITHM, $ciphertext_raw, $key, true);
			$ciphertext = base64_encode( $iv . $hmac . $ciphertext_raw );

			return $ciphertext;

		} else if (extension_loaded('mcrypt')) {
			$iv_size = mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB);
			$iv = mcrypt_create_iv($iv_size, MCRYPT_RAND);
			$h_key = hash('sha256', $key, TRUE);
			return base64_encode(mcrypt_encrypt(MCRYPT_RIJNDAEL_256, $h_key, $input_string, MCRYPT_MODE_ECB, $iv));

		} else {
			return base64_encode($input_string);
		}
	}

	private static function decrypt($encrypted_input_string, $key){
		if (extension_loaded('openssl')) {
			$c = base64_decode($encrypted_input_string);
			$ivlen = openssl_cipher_iv_length(ST_CIPHER);
			$iv = substr($c, 0, $ivlen);
			$hmac = substr($c, $ivlen, ST_SHA2LEN);
			$ciphertext_raw = substr($c, $ivlen+ST_SHA2LEN);
			$plaintext = openssl_decrypt($ciphertext_raw, ST_CIPHER, $key, OPENSSL_RAW_DATA, $iv);
			$calcmac = hash_hmac(ST_HASH_ALGORITHM, $ciphertext_raw, $key, true);
			if (hash_equals($hmac, $calcmac)) {
				return $plaintext;
			}
			return null;

		} else if (extension_loaded('mcrypt')) {
			$iv_size = mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB);
			$iv = mcrypt_create_iv($iv_size, MCRYPT_RAND);
			$h_key = hash('sha256', $key, TRUE);
			return trim(mcrypt_decrypt(MCRYPT_RIJNDAEL_256, $h_key, base64_decode($encrypted_input_string), MCRYPT_MODE_ECB, $iv));

		} else {
			return base64_decode($encrypted_input_string);
		}

	}
}
